//----------------------------------------------------------------------------
//
// Copyright (C) Sartorius Stedim Data Analytics AB 2017 -
//
// Use, modification and distribution are subject to the Boost Software
// License, Version 1.0. (See http://www.boost.org/LICENSE_1_0.txt)
//
//----------------------------------------------------------------------------

#include "stdafx.h"
#include "EzQ.h"

void CreateDummyPredictionSet(SQ_Project hProject, int iModelIndex, float*& fQuantMatrix, int& nQuantRows, int& nQuantCols, char**& szQualMatrix, int& nQualRows, int& nQualCols);
void PrintMatrix(const CEzFloatMatrix& fMatrix);
int something(char** argv);

// This implementation does not support Unicode
int main(int argc, char* argv[])
{
   if (argc != 2)
   {
      _ftprintf(stderr, _T("\n%s Usage:\n\n")
         _T(".\\SQPC++Sample.exe <USP file>\n\n")
         _T("    USP file            ---  is a SIMCA project file\n\n"),
         argv[0]);
      return 1;
   }

   // The Q object
   CEzQ oEzQ;

   // Result matrices
   CEzVectorData oTPS;
   CEzVectorData oDModXPS;
   CEzFloatMatrix fmatData;

   // Dummy prediction set variables
   float* fmatQuantitativeData = NULL;
   int nQuantRows = 0, nQuantCols = 0;
   char** szMatQualitativeData = NULL;
   int nQualRows = 0, nQualCols = 0;
   SQ_ErrorCode eError;

   // Open the project
   eError = oEzQ.OpenProject(argv[1], NULL);

   if (eError != SQ_E_OK)
   {
      printf("Could not open project. Error %s", oEzQ.GetErrorDescription(eError).c_str());
      return 0;
   }

   // Change which model to use (optional, default is first model)
   eError = oEzQ.SetModel(1);
   if (eError != SQ_E_OK)
   {
      printf("Could not open model. Error %s", oEzQ.GetErrorDescription(eError).c_str());
      return 0;
   }

   // Create dummy prediction set
   CreateDummyPredictionSet(oEzQ.GetProjectHandle(), 1, fmatQuantitativeData, nQuantRows, nQuantCols, szMatQualitativeData, nQualRows, nQualCols);

   // Predict with dummy prediction set.
   eError = oEzQ.Predict(fmatQuantitativeData, nQuantRows, nQuantCols, szMatQualitativeData, nQualRows, nQualCols);
   if (eError != SQ_E_OK)
   {
      printf("Could not predict. Error %s", oEzQ.GetErrorDescription(eError).c_str());
      return 0;
   }

   // Delete dummy prediction set
   if (fmatQuantitativeData)
      delete[] fmatQuantitativeData;
   if (szMatQualitativeData)
   {
      for (int i = 0; i < nQualCols * nQualRows; ++i)
         delete[] szMatQualitativeData[i];
      delete[] szMatQualitativeData;
   }

   // Get some results
   oEzQ.GetTPS(oTPS);
   oEzQ.GetDModXPS(oDModXPS);

   // Print results
   oTPS.GetMatrix(fmatData);
   printf("Score:\n");
   PrintMatrix(fmatData);
   oDModXPS.GetMatrix(fmatData);
   printf("DModX:\n");
   PrintMatrix(fmatData);

   return 0;
}

/**
 * Function to print all values in a matrix
 */
void PrintMatrix(const CEzFloatMatrix& fMatrix)
{
   int nRows, nCols;
   float fVal;

   SQ_GetNumRowsInFloatMatrix(fMatrix, &nRows);
   SQ_GetNumColumnsInFloatMatrix(fMatrix, &nCols);

   for (int iRow = 1; iRow <= nRows; ++iRow)
   {
      for (int iCol = 1; iCol <= nCols; ++iCol)
      {
         SQ_GetDataFromFloatMatrix(fMatrix, iRow, iCol, &fVal);
         printf("%f\t", fVal);
      }
      printf("\n");
   }
}

/**
* Function to create a dummy prediction set. In a "real" world example it is read from a file or somewhere else.
*/
void CreateDummyPredictionSet(SQ_Project hProject, int iModelIndex, float*& fQuantMatrix, int& nQuantRows, int& nQuantCols, char**& szQualMatrix, int& nQualRows, int& nQualCols)
{
   int iNumVars;
   int iModelNumber;
   SQ_Model hModel = NULL;
   SQ_PreparePrediction hPrepPred = NULL;
   SQ_VariableVector hVarVec = NULL;
   SQ_StringVector oVarNames = NULL;
   SQ_Variable hVariable = NULL;
   char szBuffer[256];
   int iNumObs = 1; // Create a prediction set with only one row

   fQuantMatrix = NULL;
   szQualMatrix = NULL;

   SQ_GetModelNumberFromIndex(hProject, iModelIndex, &iModelNumber);
   SQ_GetModel(hProject, iModelNumber, &hModel);
   SQ_GetPreparePrediction(hModel, &hPrepPred);
   SQ_GetVariablesForPrediction(hPrepPred, &hVarVec);
   SQ_GetNumVariablesInVector(hVarVec, &iNumVars);

   // Count number of qualitative and quantitative variables
   int nQualitative = 0;
   for (int iVar = 1; iVar <= iNumVars; ++iVar)
   {
      SQ_Bool bIsQual;
      SQ_GetVariableFromVector(hVarVec, iVar, &hVariable);
      SQ_IsQualitative(hVariable, &bIsQual);
      if (bIsQual == SQ_True)
         ++nQualitative;
   }

   // Create prediction data
   nQualCols = nQualitative;
   nQualRows = iNumObs;
   if (nQualitative > 0)
      szQualMatrix = new char*[nQualCols * nQualRows];

   nQuantCols = iNumVars - nQualitative;
   nQuantRows = iNumObs;
   fQuantMatrix = new float[nQuantCols * nQuantRows];

   int iQualPos = 0;
   int iQuantPos = 0;
   SQ_Bool bIsQual;

   for (int iVar = 1; iVar <= iNumVars; ++iVar)
   {
      SQ_GetVariableFromVector(hVarVec, iVar, &hVariable);
      SQ_IsQualitative(hVariable, &bIsQual);
      for (int iRow = 0; iRow < iNumObs; ++iRow)
      {
         if (bIsQual == SQ_True)
         {
            SQ_GetQualitativeSettings(hVariable, &oVarNames);
            SQ_GetStringFromVector(oVarNames, 1, szBuffer, sizeof(szBuffer)); // Just use the first qualitative setting
            szQualMatrix[iRow * nQualCols + iQualPos] = _strdup(szBuffer);
         }
         else
         {
            fQuantMatrix[iRow * nQuantCols + iQuantPos] = float(iRow + iVar); // Fill with some data
         }
      }

      if (bIsQual == SQ_True)
         iQualPos++;
      else
         iQuantPos++;
   }
}
